import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm


def show_2D(signal, title='Figure'):
	fig = plt.figure()
		
	plt.title(title)
	ax = fig.add_subplot(1,1,1, projection='3d')

	X,Y = np.meshgrid(np.arange(signal.shape[0]), np.arange(signal.shape[1]))
	Z = signal
	ax.plot_surface(X, Y, Z, rstride=4, cstride=4, alpha=0.25)
	cset = ax.contour(X, Y, Z, zdir='z', offset=-20, cmap=cm.coolwarm)
	cset = ax.contour(X, Y, Z, zdir='x', offset=-5, cmap=cm.coolwarm)
	cset = ax.contour(X, Y, Z, zdir='y', offset=signal.shape[0]+20, cmap=cm.coolwarm)

	ax.set_xlim3d(-5, signal.shape[1]);
	ax.set_ylim3d(0, signal.shape[0]+20);
	ax.set_zlim3d(np.min(signal)-20, np.max(signal));
	ax.set_title(title)
	
	
def create_2D_mire(n,m=-1):
	if m == -1:
		m = n
	mire = np.zeros((m,n))
	mire[:,n/2:n] = 255
	return mire

def create_2D_porte(n):
	porte = np.zeros((n,n))
	porte[:,n/4:3*n/4] = 255
	return porte
	

def create_2D_triangle(n, step=1):
	N=n*int(1/step)
	tri = np.zeros((N,N))
	tri[:,0:N/2] = np.arange(0,n/2,step)
	tri[:,N/2:N]=-1*np.arange(-n/2,0,step)
	return tri


def zero_pad_freq(tf, zoom_factor=2, duplicate_nyquist=False):
	# shift the signal as it is easier to fill new frequencies then
	tf_s = np.fft.fftshift(tf)
	
	# initialize data
	n,m = tf_s.shape
	N = zoom_factor*n
	M = zoom_factor*m
	zero_padded_tf_s = np.zeros((N,M),dtype=np.complex)
	
	# not the same process if even or odd number of samples
	if n%2 == 0:
		Nyquist_freq = tf_s[n/2][0]
		if duplicate_nyquist:
			print('Nyquist_freq = {}'.format(Nyquist_freq))
			zero_padded_tf_s += Nyquist_freq
		zero_padded_tf_s[N/2-n/2:N/2-n/2+n,M/2-m/2:M/2-m/2+m] = tf_s
	else:
		print('error, odd number of samples not supported')
		raise
			
	zero_padded_tf_s *= zoom_factor**2
	if duplicate_nyquist:
		v_offset = Nyquist_freq - Nyquist_freq * zoom_factor**2
		zero_padded_tf_s += v_offset
	# return the shifted back spectrum
	return np.fft.ifftshift(zero_padded_tf_s)
	
		
def interp2d(img_src, zoom_factor): 
    # Calcul de la dimension de l'image cible
	M,N = img_src.shape
	Mzoom = M*zoom_factor
	Nzoom = N*zoom_factor
	img_cible = np.ones((Mzoom, Nzoom), dtype=np.complex)

    # Pour chaque partie fractionnaire possible, 
    # calcul du noyau de convolution lineaire et ajout d'une marge au signal
	BLN_kernel = np.zeros((2,2))
	img_src_mirror = np.zeros((M+1, N+1), dtype=np.complex)
	img_src_mirror[0:M,0:N] = img_src
	img_src_mirror[M,:] =  img_src_mirror[M-1,:]
	img_src_mirror[:,N] =  img_src_mirror[:,N-1]
	for Fx in range(zoom_factor):
		for Fy in range(zoom_factor):
			BLN_kernel[0,0] = (1-Fx/np.float(zoom_factor))*(1-Fy/np.float(zoom_factor))
			BLN_kernel[0,1] = (1-Fx/np.float(zoom_factor))*(Fy/np.float(zoom_factor))
			BLN_kernel[1,0] = (Fx/np.float(zoom_factor))*(1-Fy/np.float(zoom_factor))
			BLN_kernel[1,1] = (Fx/np.float(zoom_factor))*(Fy/np.float(zoom_factor))

			# convolution (le noyau BLN est deja retourne, le mode est exact avec symmetrie au bord)
			img_cible[Fx::zoom_factor, Fy::zoom_factor] = \
				img_src_mirror[0:M,0:N] * BLN_kernel[0,0] + \
				img_src_mirror[1:M+1,0:N] * BLN_kernel[1,0] + \
				img_src_mirror[0:M,1:N+1] * BLN_kernel[0,1] + \
				img_src_mirror[1:M+1,1:N+1] * BLN_kernel[1,1]

	return(img_cible)
	
	
def zoom_freq_zero_padd(signal, zoom_factor=2, duplicate_nyquist=False):
	#~ show_2D(signal, 'signal')
	tf_s = np.fft.fft2(signal)
	#~ show_2D(np.abs(tf_s), 'spectrum_tf_s')
	#~ show_2D(np.angle(tf_s), 'phase_tf_s')

	zero_pad_tf = zero_pad_freq(tf_s, zoom_factor, duplicate_nyquist=duplicate_nyquist)
	#~ show_2D(zero_pad_tf, 'zero_pad_tf')

	return np.fft.ifft2(zero_pad_tf)
	
	
def zoom_freq_img_perAndSmooth(u, zoom_factor, duplicate_nyquist=False, spectrum_plus_phase=False):
	# u est l'image originale a decomposee en u = p+s
	# p est sa partie periodique
	# s est sa partie lisse
	# La fonction renvoie les transformees de Fourier de p et s avec la frequence zero en (0,0)
	# Pour reconstituer l'image u, la somme de p et s doit etre faite sur les fft inverses complexes,
	# et non sur leurs valeurs absolues : u = abs( ifft(pf) + ifft(sf) ).

	M, N = u.shape
	# Etape 1 : Calcul d'une image v qui contient les changements d'intensite aux bords de u
	# v = v1 + v2
	v1 = np.zeros(u.shape, dtype=np.float)
	v1[0, :] = u[M-1, :] - u[0,:]
	v1[M-1, :] = u[0,:] - u[M-1, :]
	v2 = np.zeros(u.shape, dtype=np.float)
	v2[:,0] = u[:, N-1] - u[:,0]
	v2[:,N-1] = u[:,0] - u[:, N-1]
	v = v1 + v2

	# Etape 2 : Calcul de la FFT de v
	vf = np.fft.fft2(v)

	# Etape 3 : Calcul de la partie lisse de l'image
	x = np.arange(0,M)
	y = np.arange(0,N)
	xg, yg = np.meshgrid(y,x)
	cosx = 2.0*np.cos(2*np.pi*xg/np.float(M))
	cosy = 2.0*np.cos(2*np.pi*yg/np.float(N))
	sf = np.zeros((M,N), dtype=np.complex)
	#print("np.where(cosx[1:,:] + cosy[1:,:] - 4.0 == 0) = {}".format(np.where(cosx[1:,:] + cosy[1:,:] - 4.0 == 0)))
	#print("np.where(cosx[0,1:] + cosy[0,1:] -4.0 == 0) = {}".format(np.where(cosx[0,1:] + cosy[0,1:] -4.0 == 0)))
	print("cosx.shape = {}", format(cosx.shape))
	show_2D(cosx, 'cosx')
	show_2D(cosy, 'cosy')
	show_2D(cosx[1:,:] + cosy[1:,:] -4.0,'cosx[1:,:] + cosy[1:,:] -4.0')
	show_2D(cosx[0,1:] + cosy[0,1:] -4.0, 'cosx[0,1:] + cosy[0,1:] -4.0')
	
	sf[1:,:] = vf[1:,:] / (cosx[1:,:] + cosy[1:,:] -4.0)
	sf[0, 1:] = vf[0,1:] / (cosx[0,1:] + cosy[0,1:] -4.0)

	# Etape 4 : Calcul de la partie periodique de l'image
	uf = np.fft.fft2(u)
	pf = uf - sf

	# Etape 5 : Zoom par zero padding de p
	zpd_pf = zero_pad_freq(pf, zoom_factor, duplicate_nyquist)

	# Etape 6 : Zoom par simple interpolation de s
	s = np.fft.ifft2(sf)
	bln_interp_s = interp2d(s, zoom_factor)
	show_2D(bln_interp_s,'bln_interp_s')

	plt.suptitle('Periodic and smooth decomposition : spatial domain')
	plt.subplot(411)
	plt.plot(u[0,:], label='signal')
	plt.legend()
	plt.subplot(412)
	plt.plot(np.real(np.fft.ifft2(pf))[0,:], label='periodic')
	plt.legend()
	plt.subplot(413)
	plt.plot(s[0,:], label='smooth')
	plt.legend()
	plt.subplot(414)
	plt.plot(np.real(np.fft.ifft2(pf))[0,:]+s[0,:], label='periodic + smooth')
	plt.legend()
	plt.show()
		
	plt.suptitle('Periodic and smooth decomposition : freq domain')
	plt.subplot(321)
	plt.title('Spectrum')
	plt.plot(np.abs(uf[0,:]),label='tf_signal')
	plt.legend()
	plt.subplot(322)
	plt.title('Phase')
	plt.plot(np.angle(uf[0,:]), label='tf_signal')
	plt.subplot(323)
	plt.plot(np.abs(pf[0,:]),label='pf')
	plt.legend()
	plt.subplot(324)
	plt.plot(np.angle(pf[0,:]), label='pf')
	plt.subplot(325)
	plt.plot(np.abs(sf[0,:]),label='sf')
	plt.legend()
	plt.subplot(326)
	plt.plot(np.angle(sf[0,:]), label='sf')
	plt.show()


	plt.suptitle('Periodic and smooth decomposition : freq domain (zoomed)')
	plt.subplot(321)
	plt.title('Spectrum')
	plt.plot(np.abs(uf[0,:]),label='tf_signal')
	plt.legend()
	plt.subplot(322)
	plt.title('Phase')
	plt.plot(np.angle(uf[0,:]), label='tf_signal')
	plt.subplot(323)
	plt.plot(np.abs(zpd_pf[0,:]),label='z_tf_p')
	plt.legend()
	plt.subplot(324)
	plt.plot(np.angle(zpd_pf[0,:]), label='z_tf_p')
	plt.subplot(325)
	plt.plot(np.abs(np.fft.fft2(bln_interp_s)[0,:]),label='bln_interp_tf_s')
	plt.legend()
	plt.subplot(326)
	plt.plot(np.angle(np.fft.fft2(bln_interp_s)[0,:]), label='bln_interp_tf_s')
	plt.show()
	
	# Etape 5 : merge et retour en spatial
	return np.fft.ifft2(zpd_pf) + bln_interp_s
